@@ -1,12 +1,12 @@
 /* Python/gdb header for generic use in gdb
 
    Modified by Linaro.
-   Copyright (C) March 2023- Linaro Limited (or its affiliates). All rights reserved.
+   Copyright (C) March 2023-2025 Linaro Limited (or its affiliates). All rights reserved.
    
    Modified by Arm.
    Copyright (C) 1995-2023 Arm Limited (or its affiliates). All rights reserved.
    
-   Copyright (C) 2008-2022 Free Software Foundation, Inc.
+   Copyright (C) 2008-2023 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -34,6 +34,12 @@ extern const struct extension_language_defn extension_language_python;
 /* Command element for the 'python' command.  */
 extern cmd_list_element *python_cmd_element;
 
+/* The "current" objfile.  This is set when gdb detects that a new
+   objfile has been loaded.  It is only set for the duration of a call to
+   gdbpy_source_objfile_script and gdbpy_execute_objfile_script; it is NULL
+   at other times.  */
+extern struct objfile *gdbpy_current_objfile;
+
 struct value *apply_val_child (struct value *object,
 			       struct value *index,
 			       const struct language_defn *language);
